/*
 * Copyright 2009-2010 Nanjing RedOrange ltd (http://www.red-orange.cn)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package redora.client.mvp;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import redora.client.Persistable;
import redora.client.validation.BusinessRuleViolation;

/**
 * After a connect or disConnect for two objects is invoked in the service object, the service
 * object will fire an ConnectEvent when it has retrieved a response
 * from the server.
 *
 * @author Nanjing RedOrange (www.red-orange.cn)
 */
public class ConnectEvent extends GwtEvent<ConnectEvent.Handler> {
    public static Type<ConnectEvent.Handler> TYPE = new Type<ConnectEvent.Handler>();

    /**
 * Note that there is no onError event. When an error occurs, this
 * will be notified directly to the info/error message label.
 * @author Nanjing RedOrange (www.red-orange.cn)
 */
public interface Handler extends EventHandler {
    /** Deleted successfully.*/
    void onSuccess(ConnectEvent event);
}

    public enum ActionType {
        connect, disConnect
    }

    public final Class<? extends Persistable> originalCls;
    public final Class<? extends Persistable> targetCls;
    public final ActionType actionType;
    public final Long[] idParams;

    public ConnectEvent(Class<? extends Persistable> originalCls, Class<? extends Persistable> targetCls, ActionType actionType, Long[] idParams) {
        this.originalCls = originalCls;
        this.targetCls = targetCls;
        this.actionType = actionType;
        this.idParams = idParams;
    }

    public Long getOriginalPojoId() {
        if (idParams != null && idParams.length == 2)
            return idParams[0];
        return null;
    }

    public Long getTargetPojoId() {
        if (idParams != null && idParams.length == 2)
            return idParams[1];
        return null;
    }

    @Override
    public Type<ConnectEvent.Handler> getAssociatedType() {
        return TYPE;
    }

    @Override
    protected void dispatch(ConnectEvent.Handler handler) {
        handler.onSuccess(this);
    }
}


